/*
 * This file is part of the Disco Deterministic Network Calculator v2.1.0 "Hydra".
 *
 * Copyright (C) 2011 - 2015 Steffen Bondorf
 *
 * disco | Distributed Computer Systems Lab
 * University of Kaiserslautern, Germany
 *
 * http://disco.cs.uni-kl.de
 *
 *
 * The Disco Deterministic Network Calculator (DiscoDNC) is free software;
 * you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 */

package unikl.disco.demos;

import java.util.Iterator;

import unikl.disco.curves.MaxServiceCurve;
import unikl.disco.curves.ServiceCurve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.nc.AnalysisConfig;
import unikl.disco.nc.AnalysisConfig.GammaFlag;
import unikl.disco.nc.PmooAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Server;

/**
 * 
 * @author Steffen Bondorf
 */
public class Demo1
{
	public Demo1() {}

	@SuppressWarnings("unused")
	public void run()
	{
		ArrivalCurve arrival_curve = ArrivalCurve.createTokenBucket( 0.1e6, 0.1 * 0.1e6 );
		
		ServiceCurve service_curve = ServiceCurve.createRateLatency( 10.0e6, 0.01 );
		MaxServiceCurve max_service_curve = MaxServiceCurve.createRateLatency( 100.0e6, 0.001 );
		
		Network network = new Network();
		AnalysisConfig configuration = new AnalysisConfig();

		try {
			Server s0 = network.addServer( service_curve, max_service_curve );
			// Creating a server with a maximum service curve automatically triggers
//			s0.setUseGamma( true );
//			s0.setUseExtraGamma( true );
			// , however, it can be disabled globally by setting
			configuration.setUseGamma( GammaFlag.GLOBALLY_ON );
			configuration.setUseExtraGamma( GammaFlag.GLOBALLY_ON );
			
			Flow f0 = network.addFlow( arrival_curve, s0 );

			Iterator<Flow> flow_iter = network.getFlows().iterator();
			while ( flow_iter.hasNext() ) {
				Flow flow_of_interest = flow_iter.next();
				
				System.out.println( "Flow of interest : " + flow_of_interest.toString() );
				System.out.println();
		
//				Analyze the network
//				TFA
				System.out.println( "--- Total Flow Analysis ---" );
				TotalFlowAnalysis tfa = new TotalFlowAnalysis( network, configuration );
				
				try {
					tfa.performAnalysis( flow_of_interest );
					System.out.println( "delay bound     : " + tfa.getDelayBound() );
					System.out.println( "     per server : " + tfa.getServerDelayBoundMap().toString() );
					System.out.println( "backlog bound   : " + tfa.getBacklogBound() );
					System.out.println( "     per server : " + tfa.getServerBacklogBoundMap().toString() );
					System.out.println( "alpha per server: " + tfa.getServerAlphasMap().toString() );
				} catch (Exception e) {
					System.out.println( "TFA analysis failed" );
					System.out.println( e.toString() );
				}
				
				System.out.println();
			
//				SFA
				System.out.println( "--- Separated Flow Analysis ---" );
				SeparateFlowAnalysis sfa = new SeparateFlowAnalysis( network, configuration );
				
				try {
					sfa.performAnalysis( flow_of_interest );
					System.out.println( "e2e SFA SCs     : " + sfa.getLeftOverServiceCurves() );
					System.out.println( "     per server : " + sfa.getServerLeftOverBetasMap().toString() );
					System.out.println( "xtx per server  : " + sfa.getServerAlphasMap().toString() );
					System.out.println( "delay bound     : " + sfa.getDelayBound() );
					System.out.println( "backlog bound   : " + sfa.getBacklogBound() );
				} catch (Exception e) {
					System.out.println( "SFA analysis failed" );
					System.out.println( e.toString() );
				}
				
				System.out.println();
			
//				PMOO
				System.out.println( "--- PMOO Analysis ---" );
				PmooAnalysis pmoo = new PmooAnalysis( network, configuration );
				
				try {
					pmoo.performAnalysis( flow_of_interest );
					System.out.println( "e2e PMOO SCs    : " + pmoo.getLeftOverServiceCurves() );
					System.out.println( "xtx per server  : " + pmoo.getServerAlphasMap().toString() );
					System.out.println( "delay bound     : " + pmoo.getDelayBound() );
					System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
				} catch (Exception e) {
					System.out.println( "PMOO SFA analysis failed" );
					System.out.println( e.toString() );
				}

				System.out.println();
				System.out.println();
			}
		} catch (Exception e) {
			System.out.println( e.toString() );
		}
	}

	public static void main( String[] args )
	{
		Demo1 demo = new Demo1();
		demo.run();
	}
}
